//Kurzzeitwecker.cpp : source file for the Kurzzeitwecker project
//

#include "Kurzzeitwecker.h"

/////////////////////////////////////////////////////////////////////////////
//Kurzzeitwecker

	

// Bitmasken fr Ziffern 0..9 entsprechend der Segmente a..g
static short digit_bitmask[] = {0b00111111, 0b00000110, 0b01011011, 0b01001111,
	                            0b01100110, 0b01101101, 0b01111101, 0b00000111,
	                            0b01111111, 0b01101111};
// static short sinwavelen[] = {0,2,6,14,24,37,53,70,88,108,128,147,167,185,202,218,231,241,249,253,255};

// bei Festhalten des Tasters wird die Schrittgeschwindigkeit langsam erhht
static int LONGPRESS_DELAYS[] = {510, 450, 390, 330};

int d = 1;

static void io_init(void)
{
	// PortB
	PORTB = 0x0;
	DDRB = 0xff;
	// PortC
	PORTC = 0x0;
	DDRC = 0x7f;
	// PortD
	PORTD = 0x0;
	DDRD = 0b11111011;

	// Watchdog Enabled, Prescaler: OSC/16k
	//wdt_enable(WDTO_15MS);

	// Analog Comparator Disabled
	ACSR = 0x80;

	// INT0 Enabled, Mode: Low Level
	MCUCR |= 0x00;
   	GICR = 0x40;
   	  
   	// configure power down mode
   	cbi(MCUCR,4);
	sbi(MCUCR,5);
	cbi(MCUCR,6);

	// disable interrupts
	cli();
}

SIGNAL(SIG_INTERRUPT0)
{
	// nothing to do here, interrupt is only used to wake up
}

	/*
	Ziffer 1
	
	PC0 = Segment a = LED-PIN 1
	PC1 = Segment b = LED-PIN 14
	PC2 = Segment c = LED-PIN 12
	PC3 = Segment d = LED-PIN 10
	PC4 = Segment e = LED-PIN 4
	PC5 = Segment f = LED-PIN 2
	PB2 = Segment g = LED-PIN 13
	PB1 = Segment . = LED-PIN 9
	
	Ziffer 0 (180 gedreht)
	
	PD0 = Segment d = LED-PIN 10
	PD1 = Segment e = LED-PIN 4
	PD3 = Segment f = LED-PIN 2
	PD4 = Segment a = LED-PIN 1
	PD5 = Segment b = LED-PIN 14
	PD6 = Segment c = LED-PIN 12
	PD7 = Segment g = LED-PIN 13
	
	Buzzer
	
	PB0 = Buzzer
	
	Button
	
	PD2 = Button
	*/

void led1_out_bitmask(short bitmask)
{
	// set digit 0 segments a-f
	PORTC = (PORTC & 0b11000000) | (bitmask & 0b00111111);
	// set digit 0 segment g
	PORTB = (PORTB & 0b11111011) | ((bitmask & 0b01000000) >> 4);
}

void led0_out_bitmask(short bitmask)
{
	// set digit 1 segments a-b
	PORTD = (PORTD & 0b11111100) | (bitmask & 0b00000011);
	// set digit 1 segments c-g
	PORTD = (PORTD & 0b00000111) | ((bitmask & 0b01111100) << 1);
}

void led_out(int i)
{
	if (i > 99) { i = 99; }
	if (i < 00) { i =  0; }
	led0_out_bitmask(digit_bitmask[i % 10]);
	if (i / 10)
	{
		led1_out_bitmask(digit_bitmask[i / 10]);
 	}
 	else
 	{
		led1_out_bitmask(0);
 	}
}

void clear_display(void)
{
	led0_out_bitmask(0);
	led1_out_bitmask(0);
}

// go to sleep and wait until the user wakes up by pressing the button
void go_to_sleep()
{
  // clear display
  clear_display();
  
  sei(); // enable interrupts to make wakeup possible
  sbi(MCUCR, SE); // Sleep enable bit makes "sleep" asm instruction available
  asm volatile("sleep"); // go to sleep
  
  // after wakeup
  
  cbi(MCUCR, SE); // disable sleep instruction
  cli(); // disable interrupts
}

void buzzer(bool on)
{
	if (on)
	{
		sbi(PORTB, 0);
	}
	else
	{
		cbi(PORTB, 0);
	}
}

void point(bool on)
{
	if (on)
	{
		sbi(PORTB, 1);
	}
	else
	{
		cbi(PORTB, 1);
	}
}

void sinsweep(int freq1, int freq2, long length)
{
	long time = 0;
	int freq;
	long pw;
	int q = freq2 - freq1;
	
	while (time < length)
	{
		freq = freq1 + (q * time / length);
		pw   = 500000 / freq;
		
		buzzer(true);
		sleepus(pw);
		buzzer(false);
		sleepus(pw);
		time += pw * 2;
	}
}

void sinsound(int freq, long len)
{
	long pw = 1000000 / freq / 2;
	int loops = len / pw / 2;
	int i;
	
	for (i = 0; i < loops; i++)
	{
		buzzer(true);
		sleepus(pw);
		buzzer(false);
		sleepus(pw);
	}
}

inline void sound_beep()
{
  sinsound(500, 20000);
}

inline void sound_beep2()
{
  sinsound(800, 20000);
}

inline void sound_shutdown()
{
  sinsound(4000, 50000);
  sleep(50);
  sinsound(3000, 50000);
  sleep(50);
  sinsound(2000, 50000);
}

inline void sound_startcountdown()
{
  sinsound(2000, 50000);
  sleep(50);
  sinsound(3000, 50000);
  sleep(50);
  sinsound(4000, 50000);
}

inline void sound_click()
{
    buzzer(true); // uncomment to enable a clicking noise in the last minute
    sleepus(5);
	buzzer(false); // uncomment to enable a clicking noise in the last minute	
}

inline void sound_alarm()
{
  int i;
  
  for (i = 0; i < 2; i++)
  {
  	led_out(0);
    sinsound(4000, 50000);
    sleep(200);
    clear_display();
    sleep(250);
  	led_out(0);
    sleep(250);
    clear_display();
    sleep(250);
  }

  for (i = 0; i < 4; i++)
  {
  	led_out(0);
    sinsound(4000, 50000);
    sleep(200);
    clear_display();
    sleep(250);
  }

  led_out(0);
  sinsound(4000, 1000000);
}

void demo(void)
{
	int i;
	
	sleep(1000);

	for (i = 99; i >= 0; i--)
	{
		led_out(i);
		sleep(18);
	}

	sleep(1000);
}

// let the user enter the time
short get_time(short start)
{
	short res = start;
	int c = 0; // count how long no user interaction happened
	led_out(res);
	short i;
	
	sound_beep();
	
	while (c < TIMEOUT_NOPRESS_MS)
	{
		// set minutes 5-minute-wise with holding down the button
		sleep(60);
		c = 0;
		while (BUTTON_ACTIVE && c < LONGPRESS_DELAYS[0]) {sleep(10); c += 10;}
		if (BUTTON_ACTIVE)
		{
			short tick = 0;
			while (BUTTON_ACTIVE)
			{
				sound_beep2();
				res = ((res / 5) * 5 + 5) % 100;
				led_out(res);
				// make 10 single delays instead of one big delay
				// to detect intermediate button release and button press afterwards
				for (i = 0; i < 10; i++)
				{
					sleep(LONGPRESS_DELAYS[tick] / 10);
					if (!BUTTON_ACTIVE)
					{
						break;
					}
				}
				if (tick < 3) tick++;
			}
		}

		// set minutes 1-minute-wise with single button presses
		sleep(60);
		c = 0;
		while (!BUTTON_ACTIVE && (c < TIMEOUT_NOPRESS_MS)) { sleep(10); c += 10; }
		if (BUTTON_ACTIVE)
		{
            sound_beep();
			res = (res + 1) % 100;
			led_out(res);
		}
	}
	
	// make three little beeps when a time was selected
	// and one longer beep when the timer was disabled by selecting "0"
	if (res > 0)
	{
		sound_startcountdown();		
	}
	else
	{
		sound_shutdown();		
	}
	
	return res;
}

void countdown(int time)
{
	int sec;
	int sub;
	bool user_disable = time == 0;
	
	sec = time * 60;
	
	while (sec > 1)
	{
		sec--;

		if (sec > 60)
		{
			led_out((sec / 60) + 1);
		}
		else
		{
			led_out(sec);
			// sound_click(); // comment out to hear clicking noise
		}

		point(true);
		for (sub = 0; sub < 10; sub++)
		{
			if (BUTTON_ACTIVE) break;
			sleep(50);
		}
		point(false);
		for (sub = 0; sub < 10; sub++)
		{
			if (BUTTON_ACTIVE) break;
			sleep(50);
		}
		
		// user can interrupt and change the countdown time
		if (BUTTON_ACTIVE)
		{
			sec = get_time((sec / 60) + 1) * 60;
			if (sec == 0) // user disabled timer
			{
				user_disable = true;
				break;	
			}
		}
	}

	if (!user_disable)
	{
		sound_alarm();
	}
}

int main(void)
{
	io_init();
    demo();
	sound_shutdown();
			
	int time;
	
	// main loop
	while(1)
	{
		go_to_sleep();
		time = get_time(1);
		countdown(time);
	}
}
